@@ -1,7 +1,7 @@
 /*
  * This file is part of Tepl, a text editor library.
  *
- * Copyright 2016 - Sébastien Wilmet <swilmet@gnome.org>
+ * Copyright 2016, 2017 - Sébastien Wilmet <swilmet@gnome.org>
  *
  * Tepl is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Lesser General Public License as published by the
@@ -18,6 +18,7 @@
  */
 
 #include "tepl-buffer.h"
+#include "tepl-abstract-factory.h"
 #include "tepl-file.h"
 #include "tepl-utils.h"
 
@@ -52,7 +53,8 @@ struct _TeplBufferPrivate
 enum
 {
 	PROP_0,
-	PROP_TEPL_TITLE,
+	PROP_TEPL_SHORT_TITLE,
+	PROP_TEPL_FULL_TITLE,
 	PROP_TEPL_STYLE_SCHEME_ID,
 	N_PROPERTIES
 };
@@ -102,8 +104,12 @@ tepl_buffer_get_property (GObject    *object,
 
 	switch (prop_id)
 	{
-		case PROP_TEPL_TITLE:
-			g_value_take_string (value, tepl_buffer_get_title (buffer));
+		case PROP_TEPL_SHORT_TITLE:
+			g_value_take_string (value, tepl_buffer_get_short_title (buffer));
+			break;
+
+		case PROP_TEPL_FULL_TITLE:
+			g_value_take_string (value, tepl_buffer_get_full_title (buffer));
 			break;
 
 		case PROP_TEPL_STYLE_SCHEME_ID:
@@ -259,7 +265,8 @@ tepl_buffer_modified_changed (GtkTextBuffer *buffer)
 		GTK_TEXT_BUFFER_CLASS (tepl_buffer_parent_class)->modified_changed (buffer);
 	}
 
-	g_object_notify_by_pspec (G_OBJECT (buffer), properties[PROP_TEPL_TITLE]);
+	g_object_notify_by_pspec (G_OBJECT (buffer), properties[PROP_TEPL_SHORT_TITLE]);
+	g_object_notify_by_pspec (G_OBJECT (buffer), properties[PROP_TEPL_FULL_TITLE]);
 }
 
 static void
@@ -279,15 +286,30 @@ tepl_buffer_class_init (TeplBufferClass *klass)
 	text_buffer_class->modified_changed = tepl_buffer_modified_changed;
 
 	/**
-	 * TeplBuffer:tepl-title:
+	 * TeplBuffer:tepl-short-title:
 	 *
-	 * The buffer title. See tepl_buffer_get_title().
+	 * The short title. See tepl_buffer_get_short_title().
 	 *
-	 * Since: 2.0
+	 * Since: 3.0
 	 */
-	properties[PROP_TEPL_TITLE] =
-		g_param_spec_string ("tepl-title",
-				     "Tepl Title",
+	properties[PROP_TEPL_SHORT_TITLE] =
+		g_param_spec_string ("tepl-short-title",
+				     "tepl-short-title",
+				     "",
+				     NULL,
+				     G_PARAM_READABLE |
+				     G_PARAM_STATIC_STRINGS);
+
+	/**
+	 * TeplBuffer:tepl-full-title:
+	 *
+	 * The full title. See tepl_buffer_get_full_title().
+	 *
+	 * Since: 3.0
+	 */
+	properties[PROP_TEPL_FULL_TITLE] =
+		g_param_spec_string ("tepl-full-title",
+				     "tepl-full-title",
 				     "",
 				     NULL,
 				     G_PARAM_READABLE |
@@ -336,11 +358,12 @@ tepl_buffer_class_init (TeplBufferClass *klass)
 }
 
 static void
-short_name_notify_cb (TeplFile   *file,
-		      GParamSpec *pspec,
-		      TeplBuffer *buffer)
+file_short_name_notify_cb (TeplFile   *file,
+			   GParamSpec *pspec,
+			   TeplBuffer *buffer)
 {
-	g_object_notify_by_pspec (G_OBJECT (buffer), properties[PROP_TEPL_TITLE]);
+	g_object_notify_by_pspec (G_OBJECT (buffer), properties[PROP_TEPL_SHORT_TITLE]);
+	g_object_notify_by_pspec (G_OBJECT (buffer), properties[PROP_TEPL_FULL_TITLE]);
 }
 
 static void
@@ -357,14 +380,16 @@ static void
 tepl_buffer_init (TeplBuffer *buffer)
 {
 	TeplBufferPrivate *priv;
+	TeplAbstractFactory *factory;
 
 	priv = tepl_buffer_get_instance_private (buffer);
 
-	priv->file = tepl_file_new ();
+	factory = tepl_abstract_factory_get_singleton ();
+	priv->file = tepl_abstract_factory_create_file (factory);
 
 	g_signal_connect_object (priv->file,
 				 "notify::short-name",
-				 G_CALLBACK (short_name_notify_cb),
+				 G_CALLBACK (file_short_name_notify_cb),
 				 buffer,
 				 0);
 
@@ -393,6 +418,8 @@ tepl_buffer_new (void)
  * Returns the #TeplFile of @buffer. The returned object is guaranteed to be the
  * same for the lifetime of @buffer.
  *
+ * #TeplBuffer creates the #TeplFile with tepl_abstract_factory_create_file().
+ *
  * Returns: (transfer none): the associated #TeplFile.
  * Since: 1.0
  */
@@ -440,40 +467,72 @@ tepl_buffer_is_untouched (TeplBuffer *buffer)
 }
 
 /**
- * tepl_buffer_get_title:
+ * tepl_buffer_get_short_title:
+ * @buffer: a #TeplBuffer.
+ *
+ * Returns a title suitable for a tab label. It contains (in that order):
+ * - '*' if the buffer is modified;
+ * - the #TeplFile:short-name;
+ *
+ * Returns: the @buffer short title. Free the return value with g_free() when no
+ * longer needed.
+ * Since: 3.0
+ */
+gchar *
+tepl_buffer_get_short_title (TeplBuffer *buffer)
+{
+	TeplBufferPrivate *priv;
+	const gchar *short_name;
+	gchar *short_title;
+
+	g_return_val_if_fail (TEPL_IS_BUFFER (buffer), NULL);
+
+	priv = tepl_buffer_get_instance_private (buffer);
+
+	short_name = tepl_file_get_short_name (priv->file);
+
+	if (gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (buffer)))
+	{
+		short_title = g_strconcat ("*", short_name, NULL);
+	}
+	else
+	{
+		short_title = g_strdup (short_name);
+	}
+
+	return short_title;
+}
+
+/**
+ * tepl_buffer_get_full_title:
  * @buffer: a #TeplBuffer.
  *
  * Returns a title suitable for a #GtkWindow title. It contains (in that order):
- * - '*' if the buffer is modified;
- * - the #TeplFile:short-name;
+ * - the #TeplBuffer:tepl-short-title;
  * - the directory path in parenthesis if the #TeplFile:location isn't
  *   %NULL.
  *
- * Returns: the @buffer title. Free the return value with g_free() when no
+ * Returns: the @buffer full title. Free the return value with g_free() when no
  * longer needed.
- * Since: 1.0
+ * Since: 3.0
  */
 gchar *
-tepl_buffer_get_title (TeplBuffer *buffer)
+tepl_buffer_get_full_title (TeplBuffer *buffer)
 {
 	TeplBufferPrivate *priv;
 	GFile *location;
-	const gchar *short_name;
-	gchar *title;
+	gchar *short_title;
+	gchar *full_title;
 
 	g_return_val_if_fail (TEPL_IS_BUFFER (buffer), NULL);
 
 	priv = tepl_buffer_get_instance_private (buffer);
 
 	location = tepl_file_get_location (priv->file);
-	short_name = tepl_file_get_short_name (priv->file);
+	short_title = tepl_buffer_get_short_title (buffer);
 
-	if (location == NULL ||
-	    !g_file_has_parent (location, NULL))
-	{
-		title = g_strdup (short_name);
-	}
-	else
+	if (location != NULL &&
+	    g_file_has_parent (location, NULL))
 	{
 		GFile *parent;
 		gchar *directory;
@@ -483,24 +542,19 @@ tepl_buffer_get_title (TeplBuffer *buffer)
 		directory = g_file_get_parse_name (parent);
 		directory_tilde = _tepl_utils_replace_home_dir_with_tilde (directory);
 
-		title = g_strdup_printf ("%s (%s)", short_name, directory_tilde);
+		full_title = g_strdup_printf ("%s (%s)", short_title, directory_tilde);
+		g_free (short_title);
 
 		g_object_unref (parent);
 		g_free (directory);
 		g_free (directory_tilde);
 	}
-
-	if (gtk_text_buffer_get_modified (GTK_TEXT_BUFFER (buffer)))
+	else
 	{
-		gchar *full_title;
-
-		full_title = g_strconcat ("*", title, NULL);
-		g_free (title);
-
-		return full_title;
+		full_title = short_title;
 	}
 
-	return title;
+	return full_title;
 }
 
 /**
