@@ -279,6 +279,9 @@ get_translated_name (const gchar *charset)
  * Creates a new #TeplEncoding from a character set such as "UTF-8" or
  * "ISO-8859-1".
  *
+ * The tepl_encoding_get_charset() function will return exactly the same string
+ * as the @charset passed in to this constructor.
+ *
  * Returns: the new #TeplEncoding. Free with tepl_encoding_free().
  * Since: 2.0
  */
@@ -352,7 +355,9 @@ tepl_encoding_get_charset (const TeplEncoding *enc)
  * tepl_encoding_get_name:
  * @enc: a #TeplEncoding.
  *
- * Gets the name of the #TeplEncoding such as "Unicode" or "Western".
+ * Gets the name of the #TeplEncoding such as "Unicode" or "Western". If the
+ * charset is unknown by #TeplEncoding, "Unknown" is returned. The return value
+ * is already translated by gettext.
  *
  * Returns: the name of the #TeplEncoding.
  * Since: 2.0
@@ -370,7 +375,8 @@ tepl_encoding_get_name (const TeplEncoding *enc)
  * @enc: a #TeplEncoding.
  *
  * Returns the encoding name with the charset in parenthesis, for example
- * "Unicode (UTF-8)". If the name is unknown, just the charset is returned.
+ * "Unicode (UTF-8)". If the name is unknown, just the charset is returned. The
+ * name is translated by gettext.
  *
  * Returns: a string representation. Free with g_free() when no longer needed.
  * Since: 2.0
@@ -393,6 +399,12 @@ tepl_encoding_to_string (const TeplEncoding *enc)
  * tepl_encoding_is_utf8:
  * @enc: a #TeplEncoding.
  *
+ * Returns whether @enc is a UTF-8 encoding.
+ *
+ * If @enc was created with tepl_encoding_new_utf8(), the charset is "UTF-8".
+ * But iconv supports other variants: "UTF8", "utf-8" and "utf8". This function
+ * returns %TRUE for all UTF-8 variants supported by iconv.
+ *
  * Returns: whether @enc is a UTF-8 encoding.
  * Since: 2.0
  */
@@ -410,7 +422,14 @@ tepl_encoding_is_utf8 (const TeplEncoding *enc)
  * @enc1: (nullable): a #TeplEncoding, or %NULL.
  * @enc2: (nullable): a #TeplEncoding, or %NULL.
  *
- * Returns: whether @enc1 and @enc2 are equals.
+ * Returns whether @enc1 and @enc2 are equal. It returns %TRUE iff:
+ * - Both @enc1 and @enc2 are %NULL;
+ * - Or both @enc1 and @enc2 have a UTF-8 charset (see
+ *   tepl_encoding_is_utf8());
+ * - Or the charsets are equal according to g_ascii_strcasecmp() (because
+ *   iconv-compatible charsets are case insensitive).
+ *
+ * Returns: whether @enc1 and @enc2 are equal.
  * Since: 2.0
  */
 gboolean
@@ -434,35 +453,6 @@ tepl_encoding_equals (const TeplEncoding *enc1,
 	return g_ascii_strcasecmp (enc1->charset, enc2->charset) == 0;
 }
 
-/**
- * tepl_encoding_get_all:
- *
- * Gets a list of all encodings known by #TeplEncoding.
- *
- * Returns: (transfer full) (element-type TeplEncoding): a list of
- * #TeplEncoding's.
- * Since: 2.0
- */
-GSList *
-tepl_encoding_get_all (void)
-{
-	GSList *list = NULL;
-	gint i;
-
-	for (i = G_N_ELEMENTS (encodings_table) - 1; i >= 0; i--)
-	{
-		const EncodingData *cur_data = encodings_table + i;
-		TeplEncoding *enc;
-
-		enc = _tepl_encoding_new_full (cur_data->charset,
-					       _(cur_data->name_to_translate));
-
-		list = g_slist_prepend (list, enc);
-	}
-
-	return list;
-}
-
 static gboolean
 present_in_list (const GSList       *list,
 		 const TeplEncoding *enc)
@@ -482,6 +472,49 @@ present_in_list (const GSList       *list,
 	return FALSE;
 }
 
+/**
+ * tepl_encoding_get_all:
+ *
+ * Gets a list of all encodings known by #TeplEncoding.
+ *
+ * Returns: (transfer full) (element-type TeplEncoding): a list of
+ * #TeplEncoding's.
+ * Since: 2.0
+ */
+GSList *
+tepl_encoding_get_all (void)
+{
+	GSList *list = NULL;
+	TeplEncoding *locale_enc;
+	gint i;
+
+	for (i = G_N_ELEMENTS (encodings_table) - 1; i >= 0; i--)
+	{
+		const EncodingData *cur_data = encodings_table + i;
+		TeplEncoding *enc;
+
+		enc = _tepl_encoding_new_full (cur_data->charset,
+					       _(cur_data->name_to_translate));
+
+		list = g_slist_prepend (list, enc);
+	}
+
+	/* Add locale encoding first. */
+
+	locale_enc = tepl_encoding_new_from_locale ();
+
+	if (present_in_list (list, locale_enc))
+	{
+		tepl_encoding_free (locale_enc);
+	}
+	else
+	{
+		list = g_slist_prepend (list, locale_enc);
+	}
+
+	return list;
+}
+
 static GSList *
 remove_duplicates_keep_first (GSList *list)
 {
@@ -567,11 +600,14 @@ _tepl_encoding_remove_duplicates (GSList                 *list,
 
 /* Returns: (transfer full) (element-type TeplEncoding). */
 static GSList *
-strv_to_list (const gchar * const *enc_str)
+default_candidates_strv_to_list (const gchar * const *enc_str)
 {
+	GSList *all_encodings;
 	GSList *list = NULL;
 	gchar **p;
 
+	all_encodings = tepl_encoding_get_all ();
+
 	for (p = (gchar **)enc_str; p != NULL && *p != NULL; p++)
 	{
 		const gchar *charset = *p;
@@ -589,13 +625,28 @@ strv_to_list (const gchar * const *enc_str)
 		if (present_in_list (list, enc))
 		{
 			tepl_encoding_free (enc);
+			continue;
 		}
-		else
+
+		/* If the translator has not translated correctly the list of
+		 * default candidate encodings, ensure that it won't trigger a
+		 * file loading error.
+		 */
+		if (!present_in_list (all_encodings, enc))
 		{
-			list = g_slist_prepend (list, enc);
+			g_warning ("TeplEncoding: unknown charset '%s', "
+				   "ignoring it for the default candidates.",
+				   charset);
+
+			tepl_encoding_free (enc);
+			continue;
 		}
+
+		list = g_slist_prepend (list, enc);
 	}
 
+	g_slist_free_full (all_encodings, (GDestroyNotify)tepl_encoding_free);
+
 	return g_slist_reverse (list);
 }
 
@@ -609,6 +660,10 @@ strv_to_list (const gchar * const *enc_str)
  * language, country and default encoding). The UTF-8 encoding and the current
  * locale encoding are guaranteed to be present in the returned list.
  *
+ * Note that the returned list doesn't contain all encodings known by
+ * #TeplEncoding, it is a limited list that contains only the encodings that
+ * have the most likelihood to fit for the current locale.
+ *
  * Returns: (transfer full) (element-type TeplEncoding): the list of default
  * candidate encodings.
  * Since: 2.0
@@ -672,7 +727,7 @@ tepl_encoding_get_default_candidates (void)
 	g_variant_ref_sink (encodings_variant);
 
 	encodings_strv = g_variant_get_strv (encodings_variant, NULL);
-	encodings_list = strv_to_list (encodings_strv);
+	encodings_list = default_candidates_strv_to_list (encodings_strv);
 	g_free ((gpointer) encodings_strv);
 
 	/* Ensure that UTF-8 and CURRENT are present. */
